#!/usr/bin/python
#encoding: utf8
"""
This file contains helper functions
"""
import re

# pick the first item that is not 0 (equal)
sort_purchase_list = lambda table,comps,key1,key2: (filter(None, [
			-cmp(*map(lambda values: values["allowed"],              (table[key1], table[key2]))),
			 cmp(*map(lambda values: values["add_cargos"],           (table[key1], table[key2]))),
			-cmp(*map(lambda values: values["traction_type"],        (table[key1], table[key2]))),
			 cmp(*map(lambda (comp, ident): comps.index(comp),       (key1, key2))),
			 cmp(*map(lambda values: values["intro_date"],           (table[key1], table[key2]))),
			 cmp(*map(lambda (comp, ident): ident,                   (key1, key2))),
		])[0])

# sort by company's index, and within each company by ident
sort_engine_list = lambda comps,(comp1, ident1),(comp2, ident2): cmp(comps.index(comp1), comps.index(comp2)) or cmp(ident1, ident2)

class Avail(object):
	def __init__(self, comp, epoch, set):
		self.comp = comp
		self.epoch = '34' if epoch in ['3','4'] else epoch
		self.set = set
	def __str__(self):
		return "avail_%s_%s_%s"%(self.comp, self.epoch, self.set)
	def __repr__(self):
		return "Avail(%s, %s, %s)"%(self.comp, self.epoch, self.set)
	def __cmp__(self, other):
		return cmp(self.comp.lower(), other.comp.lower()) or cmp(self.epoch, other.epoch) or cmp(self.set, other.set)
	def __hash__(self):
		return hash(self.comp.lower()) ^ hash(self.epoch) ^ hash(self.set)

def make_avail(avails, comps, epochs, sets):
	"""helper function creating the 'avail_x_y_z' strings"""
	if sets[0] == "spec" or sets[0] == "rem":
		return ()
	if len(comps) != len(epochs) or len(comps) != len(sets):
		raise TypeError("invalid availability")
	avail = map(lambda a,b,c: Avail(a,b,c), comps, epochs, sets)
	for a in avail:
		if a not in avails:
			avails.append(a)
	return avail

string_regexp = re.compile(r"([^{]*){STRING}(.*)")
sup_regexp = re.compile(r"([^{]*){SUP:([0-9IV]+)}(.*)")
sup = {
	"0": u"⁰",
	"1": u"¹",
	"2": u"²",
	"3": u"³",
	"4": u"⁴",
	"5": u"⁵",
	"6": u"⁶",
	"7": u"⁷",
	"8": u"⁸",
	"9": u"⁹",
	"I": u"ᴵ",
	"V": u"ⱽ",
	}
def replace_sup(string):
	ret = ""
	for c in string:
		ret += sup[c]
	return ret

def spriteset(ident, entries, color_depth, zoom_level):
	return "%s(VEH_ID(%s), %sSPRITE_FILE(_%s_%s.png)) {\n%s\n}\n"%(("spriteset" if color_depth == "8bpp" and zoom_level == "normal" else "alternative_sprites"), ident, "" if color_depth == "8bpp" and zoom_level == "normal" else "ZOOM_LEVEL_%s, COLOR_DEPTH_%s, "%(zoom_level, color_depth), color_depth, zoom_level, '\n'.join(map(str, entries)),)

def view_normal(ident, length, offset, slice_start, slice_end, offsets, color_depth, zoom_level):
	return spriteset(ident, [
	#/* | Top    */
	[  0,
	       0 + 2*slice_start + (0 if slice_start==0 else 10),
	           8,
	              51 - 2*slice_start - 2*(16-slice_end) - (0 if slice_start==0 else 10),
	                   -3- 0 + 0*slice_start + 0*(16-slice_end) + 0*offset + offsets[0],
	                                              -30+8 + (2-2)*slice_start + 0*(16-slice_end) + (0 if slice_start==0 else 10) - 2*offset + offsets[8+0]],
	#/* / -->     */
	[ 59 + 2*(16-slice_end),
	       0 + (0 if slice_start==0 else 10),
	          39 - 2*slice_start - 2*(16-slice_end) + (0 if slice_start==0 else 3),
	              35 - 1*(16-slice_end) - (0 if slice_start==0 else 10),
	                  -25- 8 + 2*slice_start + 2*(16-slice_end) + (0 if slice_end==16 else 2) + 2*offset + offsets[1],
	                                              -22+4 - 1*slice_start + 0*(16-slice_end) + (0 if slice_start==0 else 10) - 1*offset + offsets[8+1]],
	#/* - Right  */
	[202 + 4*(16-slice_end),
	       0,
	          65 - 4*slice_start - 4*(16-slice_end),
	              21,
	                  -33-16 + 4*slice_start + 4*(16-slice_end) + (0 if slice_end==16 else 1) + 4*offset + offsets[2],
	                                              -17-0 + 0*slice_start + 0*(16-slice_end) + 0*offset + offsets[8+2]],
	#/* \ -->    */
	[371 + 2*(16-slice_end),
	       0 + (0 if slice_end==16 else 10),
	          39 - 2*slice_start - 2*(16-slice_end) - (0 if slice_start==0 else 2),
	              35 - 1*(slice_start) - (0 if slice_end==16 else 10),
	                  -14- 8 + 2*slice_start + 2*(16-slice_end) + (0 if slice_end==16 else 1) + 2*offset + offsets[3],
	                                              -22-4 + 1*slice_start + (0 if slice_end==16 else 10) + 1*offset + offsets[8+3]],
	#/* | Bottom */
	[  0,
	      60 + 2*(16-slice_end) + (0 if slice_end==16 else 10),
	           8,
	              51 - 2*slice_start - 2*(16-slice_end) - (0 if slice_start==0 else 9) - (0 if slice_end==16 else 10),
	                   -3+ 0 + 0*slice_start + 0*(16-slice_end) - 0*offset + offsets[4],
	                                              -30-8 + 2*slice_start + 2*(16-slice_end) + (0 if slice_end==16 else 10) + 2*offset + offsets[8+4]],
	#/* / <--    */
	[ 59 + 2*slice_start + (0 if slice_start==0 else 2),
	      60 + (0 if slice_end==16 else 10),
	          39 - 2*slice_start - 2*(16-slice_end) - (0 if slice_start==0 else 2),
	              35 - (0 if slice_end==16 else 10),
	                  -23+ 8 + (0 if slice_start==0 else 2) - 2*offset + offsets[5],
	                                              -22-4 + 1*slice_start + (0 if slice_end==16 else 10) + 1*offset + offsets[8+5]],
	#/* - Left   */
	[202 + 4*slice_start + (0 if slice_start==0 else 1),
	      60,
	          65 - 4*slice_start - (0 if slice_start==0 else 1) - 4*(16-slice_end),
	              21,
	                  -32+16 + (0 if slice_start==0 else 1) - 4*offset + offsets[6],
	                                              -17+0 - 0*offset + offsets[8+6]],
	#/* \ <--    */
	[371 + 2*slice_start+ (0 if slice_start==0 else 1),
	      60 + (0 if slice_start==0 else 10),
	          39 - 2*slice_start - 2*(16-slice_end) - (0 if slice_start==0 else 1),
	              35 - (0 if slice_start==0 else 10),
	                  -15+ 8 + (0 if slice_start==0 else 1) - 2*offset + offsets[7],
	                                              -22+4 - 1*slice_start + (0 if slice_start==0 else 10) - 1*offset + offsets[8+7]],
	], color_depth, zoom_level)

def view_left_1(ident, length, offset, slice_start, slice_end, offsets, color_depth, zoom_level):
	return spriteset(ident, [
	[443, 60, 17, 46, -12- 0+8 + 0*offset - (length+1)/2 + offsets[0], -25+8-4 - 2*offset + (length+3)/4 + offsets[8+0]], # /*h**/
	[ 27,  0, 31, 38, -22- 8+8 + 2*offset - (length+1)/2 + offsets[1], -25+4+4 - 1*offset - (length+3)/4 + offsets[8+1]], # /*a**/
	[143,  0, 58, 25, -32-16+8 + 4*offset - (length+1)/2 + offsets[2], -20-0+4 + 0*offset - (length+3)/4 + offsets[8+2]], # /*b**/
	[327,  0, 43, 33, -13- 8-8 + 2*offset + (length+1)/2 + offsets[3], -22-4+4 + 1*offset - (length+3)/4 + offsets[8+3]], # /*c**/
	[443,  0, 17, 46,  -3+ 0-8 - 0*offset + (length+1)/2 + offsets[4], -30-8+4 + 2*offset - (length+3)/4 + offsets[8+4]], # /*d**/
	[ 27, 60, 31, 38, -15+ 8-8 - 2*offset + (length+1)/2 + offsets[5], -22-4-4 + 1*offset + (length+3)/4 + offsets[8+5]], # /*e**/  /*^*/
	[143, 60, 58, 25, -25+16-8 - 4*offset + (length+1)/2 + offsets[6], -17+0-4 - 0*offset + (length+3)/4 + offsets[8+6]], # /*f**/  /*|*/
	[327, 60, 43, 33, -22+ 8+8 - 2*offset - (length+1)/2 + offsets[7], -19+4-4 - 1*offset + (length+3)/4 + offsets[8+7]], # /*g**/  /*|*/
	], color_depth, zoom_level)

def view_left_2(ident, length, offset, slice_start, slice_end, offsets, color_depth, zoom_level):
	return spriteset(ident, [
	[  9,  0, 17, 46, -12- 0 + 0*offset + offsets[0], -30+8 - 2*offset + offsets[8+0]], # /*a**/
	[ 99,  0, 43, 33, -29- 8 + 2*offset + offsets[1], -22+4 - 1*offset + offsets[8+1]], # /*b**/
	[268,  0, 58, 25, -25-16 + 4*offset + offsets[2], -20-0 + 0*offset + offsets[8+2]], # /*c**/
	[411,  0, 31, 38,  -8- 8 + 2*offset + offsets[3], -25-4 + 1*offset + offsets[8+3]], # /*d**/
	[  9, 60, 17, 46,  -3+ 0 - 0*offset + offsets[4], -25-8 + 2*offset + offsets[8+4]], # /*e**/
	[ 99, 60, 43, 33, -19+ 8 - 2*offset + offsets[5], -18-4 + 1*offset + offsets[8+5]], # /*f**/  /*^*/
	[268, 60, 58, 25, -32+16 - 4*offset + offsets[6], -17+0 - 0*offset + offsets[8+6]], # /*g**/  /*|*/
	[411, 60, 31, 38, -15+ 8 - 2*offset + offsets[7], -22+4 - 1*offset + offsets[8+7]], # /*h**/  /*|*/
	], color_depth, zoom_level)

def view_right_1(ident, length, offset, slice_start, slice_end, offsets, color_depth, zoom_level):
	return spriteset(ident, [
	[  9,  0, 17, 46,  -3- 0-8 + 0*offset + (length+1)/2 + offsets[0], -25+8-4 - 2*offset + (length+3)/4 + offsets[8+0]], # /*e**/
	[ 99,  0, 43, 33, -19- 8-8 + 2*offset + (length+1)/2 + offsets[1], -18+4-4 - 1*offset + (length+3)/4 + offsets[8+1]], # /*f**/  /*|*/
	[268,  0, 58, 25, -32-16+8 + 4*offset - (length+1)/2 + offsets[2], -17-0-4 + 0*offset + (length+3)/4 + offsets[8+2]], # /*g**/  /*|*/
	[411,  0, 31, 38, -15- 8+8 + 2*offset - (length+1)/2 + offsets[3], -22-4-4 + 1*offset + (length+3)/4 + offsets[8+3]], # /*h**/  /*v*/
	[  9, 60, 17, 46, -12+ 0+8 - 0*offset - (length+1)/2 + offsets[4], -30-8+4 + 2*offset - (length+3)/4 + offsets[8+4]], # /*a**/
	[ 99, 60, 43, 33, -29+ 8+8 - 2*offset - (length+1)/2 + offsets[5], -22-4+4 + 1*offset - (length+3)/4 + offsets[8+5]], # /*b**/
	[268, 60, 58, 25, -25+16-8 - 4*offset + (length+1)/2 + offsets[6], -20+0+4 - 0*offset - (length+3)/4 + offsets[8+6]], # /*c**/
	[411, 60, 31, 38,  -8+ 8-8 - 2*offset + (length+1)/2 + offsets[7], -25+4+4 - 1*offset - (length+3)/4 + offsets[8+7]], # /*d**/
	], color_depth, zoom_level)

def view_right_2(ident, length, offset, slice_start, slice_end, offsets, color_depth, zoom_level):
	return spriteset(ident, [
	[443, 60, 17, 46,  -3- 0 + 0*offset + offsets[0], -30+8 - 2*offset + offsets[8+0]], # /*d**/
	[ 27,  0, 31, 38, -15- 8 + 2*offset + offsets[1], -22+4 - 1*offset + offsets[8+1]], # /*e**/  /*|*/
	[143,  0, 58, 25, -25-16 + 4*offset + offsets[2], -17-0 + 0*offset + offsets[8+2]], # /*f**/  /*|*/
	[327,  0, 43, 33, -22- 8 + 2*offset + offsets[3], -19-4 + 1*offset + offsets[8+3]], # /*g**/  /*v*/
	[443,  0, 17, 46, -12+ 0 - 0*offset + offsets[4], -25-8 + 2*offset + offsets[8+4]], # /*h**/
	[ 27, 60, 31, 38, -22+ 8 - 2*offset + offsets[5], -25-4 + 1*offset + offsets[8+5]], # /*a**/
	[143, 60, 58, 25, -32+16 - 4*offset + offsets[6], -20+0 - 0*offset + offsets[8+6]], # /*b**/
	[327, 60, 43, 33, -13+ 8 - 2*offset + offsets[7], -22+4 - 1*offset + offsets[8+7]], # /*c**/
	], color_depth, zoom_level)
