#!/usr/bin/python
#encoding: utf8
"""
This handles the autogeneration of images
"""
from error import warning

try:
	import pixa
	import Image
except ImportError:
	warning("WARNING: pixa is not installed, automatic sprite processing is disabled")

	def encode(outfile, infile, shape, colour):
		return infile
else:
	import os
	import tables

	def encode(outfile, infile, shape, colour):
		floorplan = Image.open("%s_8bpp_normal.png"%(infile))
		spritesheet = pixa.Spritesheet(floorplan.size[0], floorplan.size[1], floorplan.palette)
		spriterow = {'height' : floorplan.size[1], 'floorplan' : floorplan}
		try:
			colour = int(colour)
		except ValueError:
			try:
				colour = int(colour, 16)
			except ValueError:
				colour = tables.colours[colour]
		seq = pixa.PixaSequence(points = [(0,0,colour)]) # insert schema here
		sc = pixa.PixaSequenceCollection(sequences = { 86 : seq })
		spriterow['render_passes'] = [{'seq':sc, 'colourset':None}]
		spritesheet.render(spriterows=[spriterow])
		spritesheet.save("%s_8bpp_normal.png"%(outfile))
		return outfile
