#!/usr/bin/python
#encoding: utf8
"""
This file contains all tables and other values that are meant to be edited by users when setting up a new project or afterwards
"""


# languages for which translations are provided
langs = ["english", "german"]

# Settings for graphics (zoom levels specified as in NFO). Currently, if anything is added here, all files must be present in all enabled variations.
graphical_features = {
	"8bpp": ("normal",),
	"32bpp": ("normal",)
	}

# list of companies available for selection
# for each epoch:
# 	company-id (as in tracking table)
# 		internal value ID (must be globally unique)
# 		company parameter string
# 		company parameter index (starting at 2, unique per epoch)
# 		region for grouping
# 		region parameter index (same as parameter index, if only one entry)
companies = {
	"1" : {
		"bav"     : ("VAL_EPOCH1_BAVARIA",  "STR_PARAM_COMPANY_BAVARIA",      3, "GERMANY",     2),
		"pruss"   : ("VAL_EPOCH1_PRUSSIA",  "STR_PARAM_COMPANY_PRUSSIA",      4, "GERMANY",     2),
		"sax"     : ("VAL_EPOCH1_SAXONY",   "STR_PARAM_COMPANY_SAXONY",       5, "GERMANY",     2),
		"SBB"     : ("VAL_EPOCH1_SBB",      "STR_PARAM_COMPANY_SWITZERLAND2", 6, "SWITZERLAND", 6),
		"StEG"    : ("VAL_EPOCH1_STEG",     "STR_PARAM_COMPANY_AUSTRIA2",     7, "AUSTRIA",     7),
		},
	"2" : {
		"DRG"     : ("VAL_EPOCH2_DRG",      "STR_PARAM_COMPANY_GERMANY",      2, "GERMANY",     2),
		"SBB"     : ("VAL_EPOCH2_SBB",      "STR_PARAM_COMPANY_SWITZERLAND",  3, "SWITZERLAND", 3),
		"OeBB"    : ("VAL_EPOCH2_OEBB",     "STR_PARAM_COMPANY_AUSTRIA",      4, "AUSTRIA",     4),
		},
	"34" : {
		"DB"      : ("VAL_EPOCH34_DB",      "STR_PARAM_COMPANY_WEST_GERMANY", 3, "GERMANY",     2),
		"DR"      : ("VAL_EPOCH34_DR",      "STR_PARAM_COMPANY_EAST_GERMANY", 4, "GERMANY",     2),
		"SBB"     : ("VAL_EPOCH34_SBB",     "STR_PARAM_COMPANY_SWITZERLAND",  5, "SWITZERLAND", 5),
		"OeBB"    : ("VAL_EPOCH34_OEBB",    "STR_PARAM_COMPANY_AUSTRIA",      6, "AUSTRIA",     6),
		},
	"5" : {
		"DB_AG"    : ("VAL_EPOCH5_DBAG",     "STR_PARAM_COMPANY_GERMANY",      3, "GERMANY",     2),
		"privat"  : ("VAL_EPOCH5_PRIVAT",   "STR_PARAM_COMPANY_PRIVAT",       4, "GERMANY",     2),
		"SBB"     : ("VAL_EPOCH5_SBB",      "STR_PARAM_COMPANY_SWITZERLAND",  5, "SWITZERLAND", 5),
		"OeBB"    : ("VAL_EPOCH5_OEBB",     "STR_PARAM_COMPANY_AUSTRIA",      6, "AUSTRIA",     6),
		},
	}

comps = ['bav','pruss','sax','DRG','DB','DR','DB_AG'] # sorting order of the companies represented in the GRF


# map usages listed in the tracking table to their string IDs, and the special AI flag (passenger only)
usage_strings = {
	""                                : ("STR_USAGE_DUMMY",          0),
	"dummy"                           : ("STR_USAGE_DUMMY",          0),
	"light freight/shunting"          : ("STR_USAGE_LIGHT_CARGO",    0),
	"light freight"                   : ("STR_USAGE_LIGHT_CARGO",    0),
	"freight/shunting"                : ("STR_USAGE_LIGHT_CARGO",    0),
	"freight"                         : ("STR_USAGE_CARGO",          0),
	"refrigerated freight"            : ("STR_USAGE_REFRIGERATED",   0),
	"sheltered bulk"                  : ("STR_USAGE_SHELTERED",      0),
	"liquids"                         : ("STR_USAGE_LIQUIDS",        0),
	"livestock"                       : ("STR_USAGE_LIVESTOCK",      0),
	"express freight"                 : ("STR_USAGE_MAIL",           0),
	"light/heavy freight/shunting"    : ("STR_USAGE_CARGO",          0),
	"heavy freight"                   : ("STR_USAGE_HEAVY_CARGO",    0),
	"heavy freight"                   : ("STR_USAGE_HEAVY_CARGO",    0),
	"passenger/heavy freight"         : ("STR_USAGE_HEAVY_CARGO",    0),
	"universal/freight"               : ("STR_USAGE_UNIVERSAL",      0),
	"universal"                       : ("STR_USAGE_UNIVERSAL",      0),
	"passenger/freight"               : ("STR_USAGE_UNIVERSAL",      0),
	"freight/passengers"              : ("STR_USAGE_UNIVERSAL",      0),
	"express/passenger/freight"       : ("STR_USAGE_UNIVERSAL",      0),
	"passenger/universal"             : ("STR_USAGE_UNIVERSAL",      0),
	"express/freight"                 : ("STR_USAGE_UNIVERSAL",      0),
	"express"                         : ("STR_USAGE_EXPRESS",        1),
	"express/passenger"               : ("STR_USAGE_EXPRESS",        1),
	"local/express"                   : ("STR_USAGE_EXPRESS",        1),
	"passenger,express"               : ("STR_USAGE_EXPRESS",        1),
	"passenger"                       : ("STR_USAGE_PASSENGER",      1),
	"passenger/local/freight"         : ("STR_USAGE_PASSENGER",      1),
	"passenger,commuter"              : ("STR_USAGE_PASSENGER",      1),
	"local"                           : ("STR_USAGE_LOCAL",          1),
	"local (passenger)"               : ("STR_USAGE_LOCAL",          1),
	"passenger/local"                 : ("STR_USAGE_LOCAL",          1),
	"local/passenger"                 : ("STR_USAGE_LOCAL",          1),
	"local (commuter, light freight)" : ("STR_USAGE_LOCAL_CARGO",    0),
	"light freight (local)"           : ("STR_USAGE_LOCAL_CARGO",    0),
	"light freight (local, commuter)" : ("STR_USAGE_LOCAL_CARGO",    0),
	"local/light freight"             : ("STR_USAGE_LOCAL_CARGO",    0),
	"commuter"                        : ("STR_USAGE_COMMUTER",       1),
	"commuter/express"                : ("STR_USAGE_COMMUTER",       1),
	"local/commuter"                  : ("STR_USAGE_LOCAL_COMMUTER", 1),
	"commuter (local)"                : ("STR_USAGE_LOCAL_COMMUTER", 1),
	}

# TODO this is redundant to the Makefile stuff and both must be changed at once
# Must be located in src/table
tracking_table_name = 'CETS_Tracking_Table.tsv'
# This is output to the docs directory when compiling
formatted_tracking_table_name = 'CETS_engine_table.txt'
project_short_name = 'cets' # Internal ID

# TODO: include loading speed modifiers and running and purchase cost calculation formula parameters
# also on/off switches for some things
