#!/usr/bin/python
#encoding: utf8
"""
This file contains all internal tables used across the generate script
"""
# dynamic tables filled in from the tracking table
table = {}
columns = {}
avails = []
caps = []
gfx_deps = []
auto_strings = []
fixed_strings = set(["STR_ENG_DESCRIPTION", "STR_TRACK_TYPES_DEFAULT", "STR_TRACK_TYPES_CLASSES", "STR_TRACK_TYPES_SPEED"])
files = {}
slices = {}
for i in range(9,17):
	slices[i]=[]

# mapping from visual vehicle length to internal vehicle slice lengths
parts = {
	 3 : (3,),
	 4 : (4,),
	 5 : (5,),
	 6 : (6,),
	 7 : (2,3,2),
	 8 : (2,4,2),
	 9 : (3,3,3),
	10 : (3,4,3),
	11 : (3,5,3),
	12 : (4,4,4),
	13 : (4,5,4),
	14 : (4,6,4),
	15 : (4,7,4),
	16 : (4,8,4),
	}

# fine-tuning of sprite offsets for the different vehicle lengths.
# each entry contains offsets for the 5 different views,
# and 8 sprites per view in the format
# length : { view_abc : (x1, x2, x3, x4, x5, x6, x7, x8,
#                        y1, y2, y3, y4, y5, y6, y7, y8) }
#
# view_normal:  default vehicle view (straight rail, depot, purchase)
# view_left_1:  first  step of a  left turn - leading vehicle turned, center vehicle not turned, back vehicle not turned
# view_left_2:  second step of a  left turn - leading vehicle turned, center vehicle     turned, back vehicle not turned
# view_right_1: first  step of a right turn - leading vehicle turned, center vehicle not turned, back vehicle not turned
# view_right_2: second step of a right turn - leading vehicle turned, center vehicle     turned, back vehicle not turned
offsets = {
	3 : {
		"view_normal"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_left_1"  : (0, 0, 0, 0, 0, 0, 0,-1,
		                 -3,-1, 0, 0, 2, 2, 0, 0),
		"view_left_2"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_right_1" : (0, 0, 0, 0, 0, 0, 0, 0,
		                 -3, 0, 0, 0, 2, 0, 0,-1),
		"view_right_2" : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		},
	4 : {
		"view_normal"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_left_1"  : (0, 0, 0, 0, 0, 0, 0,-1,
		                 -3,-1, 0, 0, 2, 2, 0, 0),
		"view_left_2"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_right_1" : (0, 0, 0, 0, 0, 0, 0, 0,
		                 -3, 0, 0, 0, 2, 0, 0,-1),
		"view_right_2" : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		},
	5 : {
		"view_normal"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_left_1"  : (0, 0, 0, 0, 0, 0, 0,-1,
		                 -3,-1, 0, 0, 2, 2, 0, 0),
		"view_left_2"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_right_1" : (0, 0, 0, 0, 0, 0, 0, 0,
		                 -3, 0, 0, 0, 2, 0, 0,-1),
		"view_right_2" : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		},
	6 : {
		"view_normal"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_left_1"  : (0, 0, 0, 0, 0, 0, 1,-1,
		                 -3,-1, 0, 0, 3, 1, 0, 0),
		"view_left_2"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0,-2, 0, 0),
		"view_right_1" : (0, 0, 0, 0, 0, 0, 0, 0,
		                 -4,-1, 0, 1, 3, 0, 0,-1),
		"view_right_2" : (0, 0, 0,-1, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		},
	7 : {
		"view_normal"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_left_1"  : (0, 0, 0, 0, 0, 0, 0,-1,
		                 -2,-1, 0, 0, 0, 1, 0, 0),
		"view_left_2"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_right_1" : (0, 0, 0, 0, 0, 0, 0, 0,
		                 -2,-1, 0, 1, 2, 0, 0,-1),
		"view_right_2" : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		},
	8 : {
		"view_normal"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_left_1"  : (0, 0, 0, 0, 0, 0, 0,-1,
		                 -2,-1, 0, 0, 0, 1, 0, 0),
		"view_left_2"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_right_1" : (0, 0, 0, 0, 0, 0, 0, 0,
		                 -2,-1, 0, 1, 2, 0, 0,-1),
		"view_right_2" : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		},
	9 : {
		"view_normal"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_left_1"  : (0, 0, 0, 0, 0, 0, 0,-1,
		                 -2,-1, 0, 0, 0, 1, 0, 0),
		"view_left_2"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_right_1" : (0, 0, 0, 0, 0, 0, 0, 0,
		                 -2,-1, 0, 1, 2, 0, 0,-1),
		"view_right_2" : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		},
	10 : {
		"view_normal"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_left_1"  : (0, 0, 0, 0, 0, 0, 0,-1,
		                 -2, 0, 0, 0, 2, 0, 0, 0),
		"view_left_2"  : (0, 0, 0, 0, 0,-1, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_right_1" : (0, 0, 0, 0, 0, 0, 0, 0,
		                 -1,-1, 0, 0, 2, 0, 0, 0),
		"view_right_2" : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		},
	11 : {
		"view_normal"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_left_1"  : (0, 0, 0, 0, 0, 0, 0,-1,
		                 -1,-1, 0, 0, 1, 1, 0, 1),
		"view_left_2"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_right_1" : (0, 0, 0, 0, 0, 0, 0, 0,
		                 -1, 0, 0, 0, 1, 0, 0, 0),
		"view_right_2" : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		},
	12 : {
		"view_normal"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_left_1"  : (0, 0, 0, 0, 0, 0, 0,-1,
		                 -1,-1, 0, 0, 1, 1, 0, 1),
		"view_left_2"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_right_1" : (0, 0, 0, 0, 0, 0, 0, 0,
		                 -1, 0, 0, 0, 1, 0, 0, 0),
		"view_right_2" : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		},
	13 : {
		"view_normal"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_left_1"  : (0, 0, 0, 0, 0, 0, 0,-1,
		                 -1,-1, 0, 0, 1, 1, 0, 1),
		"view_left_2"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_right_1" : (0, 0, 0, 0, 0, 0, 0, 0,
		                 -1, 0, 0, 0, 1, 0, 0, 0),
		"view_right_2" : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		},
	14 : {
		"view_normal"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_left_1"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_left_2"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_right_1" : (0, 0, 0, 0, 0, 0, 0, 1,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_right_2" : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		},
	15 : {
		"view_normal"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_left_1"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_left_2"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_right_1" : (0, 0, 0, 0, 0, 0, 0, 1,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_right_2" : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		},
	16 : {
		"view_normal"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_left_1"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_left_2"  : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_right_1" : (0, 0, 0, 0, 0, 0, 0, 1,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		"view_right_2" : (0, 0, 0, 0, 0, 0, 0, 0,
		                  0, 0, 0, 0, 0, 0, 0, 0),
		},
	}

# translate cargo classes from short entries in the tracking table
# to actually nml-understandable ones. all entries not in this list are
# presumed to be cargo types
cargo_classes = {
	"pass"  : "CC_PASSENGERS",
	"mail"  : "CC_MAIL",
	"expr"  : "CC_EXPRESS",
	"armo"  : "CC_ARMOURED",
	"bulk"  : "CC_BULK",
	"piece" : "CC_PIECE_GOODS",
	"liqu"  : "CC_LIQUID",
	"refg"  : "CC_REFRIGERATED",
	"haza"  : "CC_HAZARDOUS",
	"shelt" : "CC_COVERED",
	"overs" : "CC_OVERSIZED",
	"spec"  : "CC_SPECIAL",
	"npour" : "CC_NON_POURABLE",
	"powd"  : "CC_POWDERIZED",
	}

colours = {
	}

recolour_maps = [
	"grai",
	"sulp",
	"sand",
	"aore",
	"core",
	"fert",
	"iore",
	"clay",
	"olsd",
	"lime",
	"pota",
	"grvl",
	"coal",
	"frut",
	"scmt",
	"blue",
]

