#!/usr/bin/python
#encoding: utf8
"""
this file contains the tree that will contain the parsing results
"""

from functools import partial

import tables
import project_config
import img

gfx = {}

def translate_id(value):
	if value.startswith('>='):
		return '%s..4294967295'%(value[2:],)
	if value.startswith('>'):
		return '%s+1..4294967295'%(value[1:],)
	if value.startswith('<='):
		return '0..%s-1'%(value[2:],)
	if value.startswith('<'):
		return '0..%s'%(value[1:],)
	return value

def translate_date_short(value):
	if value == 'default':
		return value
	if value.startswith("date"):
		return value
	if value.startswith('>='):
		return 'date(%s,1,1)..date(2090,12,31)'%(value[2:],)
	if value.startswith('<='):
		return 'date(1920,1,1)..date(%s,12,31)'%(value[2:],)
	if value.startswith('>'):
		return 'date(%s+1,1,1)..date(2090,12,31)'%(value[1:],)
	if value.startswith('<'):
		return 'date(1920,1,1)..date(%s-1,12,31)'%(value[1:],)
	if '..' in value:
		return 'date(%s,1,1)..date(%s,12,31)'%tuple(value.split('..'))
	return 'date(%s,1,1)..date(%s,12,31)'%(value, value)

def translate_date_long(value):
	if value == 'default':
		return value
	if value.startswith("date"):
		return value
	if value.startswith('>='):
		return 'date(%s,1,1)..date(5000000,12,31)'%(value[2:],)
	if value.startswith('<='):
		return 'date(0,1,1)..date(%s,12,31)'%(value[2:],)
	if value.startswith('>'):
		return 'date(%s+1,1,1)..date(5000000,12,31)'%(value[1:],)
	if value.startswith('<'):
		return 'date(0,1,1)..date(%s-1,12,31)'%(value[1:],)
	if '..' in value:
		return 'date(%s,1,1)..date(%s,12,31)'%tuple(value.split('..'))
	return 'date(%s,1,1)..date(%s,12,31)'%(value, value)

def translate_company(epoch, value):
	if value == 'default':
		return value
	return project_config.companies[epoch][value][0]

class Switch(object):
	switch = "switch"
	ref = "SELF"
	def __init__(self, name, children, var, translate = translate_id, default = None, offset = 0):
		self.name = name
		self.var = var
		self.offset = offset
		self.children = []
		for value, target in map(lambda a: (translate(a[0]), a[1]), children):
			if value != "default" and target is not None:
				self.children.append((value, target))
			elif value == "default" and target is not None:
				default = target
		self.default = default
	__str__  = lambda self: self.name
	__repr__ = lambda self: "%s(%r, %r, %r, %r, %r)"%(self.__class__.__name__, self.name, self.children, self.var, translate_id, self.default)
	def write_switch(self, file):
		file.write('%s(FEAT_TRAINS, %s, %s, %s) {\n'%(self.switch, self.ref, self.name, self.var))
		for value, target in self.children:
			file.write('	%s : %s; \n'%(value, target))
		if self.default is not None:
			file.write('	%s; \n'%(self.default,))
		file.write('}\n')
		file.write('\n')
		return self

class ParentSwitch(Switch):
	ref = "PARENT"

class RandomSwitch(Switch):
	switch = "random_switch"
	def __init__(self, name, children, var, translate = translate_id, default = None, offset = 0):
		Switch.__init__(self, name, children, var, translate, default, offset)
		if self.offset < 0:
			self.ref = "FORWARD_SELF(%d)"%(-self.offset,)
		if self.offset > 0:
			self.ref = "BACKWARD_SELF(%d)"%(self.offset,)

class ParentRandomSwitch(Switch):
	switch = "random_switch"
	ref = "PARENT"

class Tree(object):
	def __init__(self, value = None, *args):
		self.value = value
		self.children = []
		for arg in args:
			self.children.append(arg)
	__repr__ = lambda self: '%s(%r, %s)'%(self.__class__.__name__, self.value, ', '.join(map(repr, self.children)))
	def write(self, file, prefix, filename = "", template = "", recolour = False, slice = None, postfix = ""):
		children = []
		for i, (value, target) in enumerate(self.children):
			children.append((value, target.write(file, prefix, filename, template, recolour, slice, postfix + '_%d'%(i,))))
		translate = translate_id
		purchase = self.value
		veh_slice = -slice
		slice_var = "[STORE_TEMP(%d, 0x10F), var[0x61, 0, 0xFFFFFFFF, %s]]"%(veh_slice, self.value,)
		switch = Switch
		if self.value == "date":
			switch = ParentSwitch
			translate = translate_date_long
			self.value = "date_of_last_service - 4*var[0xFA, 0, 0x000000FF]"
			slice_var = "date_of_last_service - 4*var[0xFA, 0, 0x000000FF]"
			randomized = switch('%sgfx_switch_randomized%s'%(prefix, postfix), map(lambda x: (x[0], x[1][0]), children), self.value if veh_slice == 0 else slice_var, translate, offset = veh_slice).write_switch(file)
			self.value = "date_of_last_service"
			slice_var = "date_of_last_service"
			fixed      = switch('%sgfx_switch_fixed%s'%(prefix, postfix), map(lambda x: (x[0], x[1][0]), children), self.value if veh_slice == 0 else slice_var, translate, offset = veh_slice).write_switch(file)
			purchase = "current_date"
			return (
				Switch('%sgfx_switch%s'%(prefix, postfix), [("0..1024", fixed)], "age_in_days", default = randomized).write_switch(file),
				switch('%sgfx_switch%spurchase'%(prefix, postfix), map(lambda x: (x[0], x[1][1]), children), purchase, translate, offset = 0).write_switch(file),
				)
		if self.value == "cargo":
			self.value = "cargo_type_in_veh"
			slice_var = "[STORE_TEMP(%d, 0x10F), var[0x61, 0, 0x000000FF, %s]]"%(veh_slice, "0x47",)
			purchase = "0xFF"
		if self.value == "load":
			self.value = "cargo_count*100/cargo_capacity"
			slice_var = "[STORE_TEMP(%d, 0x10F), var[0x61, 0, 0x0000FFFF, %s]*100/var[0x61, 0, 0x0000FFFF, %s]]"%(veh_slice, "0xBC", "0xBA")
			purchase = "0"
		if self.value.startswith("epoch"):
			epoch = self.value[5:]
			if epoch in ['3', '4']: epoch = "34"
			self.value = "param_epoch"+epoch
			slice_var = "param_epoch"+epoch
			ret = []
			purchase = "param_epoch"+epoch
			translate = partial(translate_company, epoch)
			name = '%sgfx_switch%s'%(prefix, postfix)
			for i, (postfix, var, offset) in enumerate((('', self.value if veh_slice == 0 else slice_var, veh_slice), ("purchase",purchase, 0))):
				if postfix == "slice" and slice is None:
					ret.append(None)
					continue
				rand = RandomSwitch('%srandom%s'%(name, postfix), map(lambda x: (x[0], x[1][i]), children), "0", lambda value: 1, offset = offset).write_switch(file)
				comp = Switch('%scompany%s'%(name, postfix), map(lambda x: (x[0], x[1][i]), children), var, translate, rand, offset = offset).write_switch(file)
				ret.append(Switch('%s%s'%(name, postfix), [('VAL_EPOCH_NONE..VAL_EPOCH_ALL', rand)], var, default = comp, offset = offset).write_switch(file))
			return ret
		if self.value in ("random", "parent_random"):
			switch = RandomSwitch if self.value == "random" else ParentRandomSwitch
			self.value = "0" # triggers
			purchase = "0"
			slice_var = "0"
		return (
			switch('%sgfx_switch%s'%(prefix, postfix), map(lambda x: (x[0], x[1][0]), children), self.value if veh_slice == 0 else slice_var, translate, offset = veh_slice).write_switch(file),
			switch('%sgfx_switch%spurchase'%(prefix, postfix), map(lambda x: (x[0], x[1][1]), children), purchase, translate, offset = 0).write_switch(file),
			)

class Leaf(object):
	def __init__(self, value):
		self.value = value
	__repr__ = lambda self: 'Leaf(%r)'%(self.value,)
	def write(self, file, prefix, filename = "", template = "", recolour = False, slice = None, postfix = ""):
		if recolour:
			return (
				"return recolour_maps + %d"%(tables.recolour_maps.index(self.value),),
				"return recolour_maps + %d"%(tables.recolour_maps.index(self.value),),
				)
		gfx_switch, purchase_switch = gfx.get((filename, self.value, slice), (None, None))
		if gfx_switch is None:
			file_name_prefix = 'src/gfx/%s_%s'%(filename, self.value)
			file.write('#define SPRITE_FILE(p) QUOTED_STRING(%s ## p)\n'%(file_name_prefix,))
			for color_depth in project_config.graphical_features:
				for zoom_level in project_config.graphical_features[color_depth]:
					if '%s_%s_%s.png'%(file_name_prefix,color_depth,zoom_level) not in tables.gfx_deps:
						tables.gfx_deps.append('%s_%s_%s.png'%(file_name_prefix,color_depth,zoom_level))
			gfx_switch = '%sgfx_switch%s'%(prefix, postfix)
			purchase_switch = '%sgui_group%s'%(prefix, postfix)
			gfx[(filename, self.value, slice)] = (gfx_switch, purchase_switch)
			if postfix:
				file.write('#define VEH_ID(...) %s ## __VA_ARGS__ ## %s\n'%(prefix, postfix))
			else:
				file.write('#define VEH_ID(...) %s ## __VA_ARGS__\n'%(prefix,))
			file.write('#define SLICE %s\n'%(slice,))
			file.write('#include "src/%s"\n'%(template,))
			file.write('#undef SLICE\n')
			file.write('#undef VEH_ID\n')
			file.write('#undef SPRITE_FILE\n')
		file.write('\n')
		return (gfx_switch, purchase_switch)

class Function(object):
	def __init__(self, value, args):
		self.value = value
		self.children = args
	__repr__ = lambda self: 'Function(%s, %s)'%(self.value, self.children)
	def write(self, file, prefix, filename = "", template = "", recolour = False, slice = None, postfix = ""):
		assert self.value in ('livery',)
		if self.value == "livery":
			gfx_switch, purchase_switch = gfx.get((filename, self.children["name"], slice), (None, None))
			if gfx_switch is None:
				file_name_prefix = img.encode('src/gfx/%s_%s'%(filename, self.children["name"]), "src/gfx/template_%s"%(filename.split('_')[-1],), self.children["shape"], self.children["colour"])
				file.write('#define SPRITE_FILE(p) QUOTED_STRING(%s ## p)\n'%(file_name_prefix,))
				for color_depth in project_config.graphical_features:
					for zoom_level in project_config.graphical_features[color_depth]:
						if '%s_%s_%s.png'%(file_name_prefix,color_depth,zoom_level) not in tables.gfx_deps:
							tables.gfx_deps.append('%s_%s_%s.png'%(file_name_prefix,color_depth,zoom_level))
				gfx_switch = '%sgfx_switch%s'%(prefix, postfix)
				purchase_switch = '%sgui_group%s'%(prefix, postfix)
				gfx[(filename, self.children["name"], slice)] = (gfx_switch, purchase_switch)
				if postfix:
					file.write('#define VEH_ID(...) %s ## __VA_ARGS__ ## %s\n'%(prefix, postfix))
				else:
					file.write('#define VEH_ID(...) %s ## __VA_ARGS__\n'%(prefix,))
				file.write('#define SLICE %s\n'%(slice,))
				file.write('#include "src/%s"\n'%(template,))
				file.write('#undef SLICE\n')
				file.write('#undef VEH_ID\n')
				file.write('#undef SPRITE_FILE\n')
		return (gfx_switch, purchase_switch)
